@@ -33,7 +33,10 @@
 #ifndef MFLASH_COMMON_STRUCTS_H
 #define MFLASH_COMMON_STRUCTS_H
 
+#include <compatibility.h>
+
 typedef struct mflash mflash;
+#define MAX_FLASH_NAME 256
 
 typedef enum MfOpt {
     MFO_NO_VERIFY = 0,
@@ -52,19 +55,16 @@ typedef enum MfOpt {
 } MfOpt;
 
 enum MfAccessType {
-    MFAT_MFILE = 0,
-    MFAT_UEFI,
+    MFAT_MFILE = 0, MFAT_UEFI,
 };
 
 typedef enum {
-    UNKNOWN_BIN = 0x0,
-    CX5_LOW_BIN  = 0x1,
-    CX5_HIGH_BIN = 0x2,
+    UNKNOWN_BIN = 0x0, CX5_LOW_BIN = 0x1, CX5_HIGH_BIN = 0x2,
 
 } BinIdT;
 
 typedef struct flash_params {
-    const char *type_name;
+    char type_name[MAX_FLASH_NAME];
     int log2size;
     int num_of_flashes;
 } flash_params_t;
@@ -75,6 +75,47 @@ typedef struct write_protect_info {
     u_int8_t sectors_num;  // if zero: is_subsector and is_bottom are invalid.
 } write_protect_info_t;
 
+////////////////////////////////////////
+//
+// ST SPI functions - common for InfiniHostIIILx and ConnectX
+//
+////////////////////////////////////////
+typedef enum StFlashCommand {
+    SFC_SE = 0xD8,
+    SFC_4SE = 0xDC,
+    SFC_SSE = 0x20,
+    SFC_4SSE = 0x21,
+    SFC_PP = 0x02,
+    SFC_4PP = 0x12,
+    SFC_READ = 0x03,
+    SFC_4READ = 0x13,
+    SFC_FAST_READ = 0x3B,
+    SFC_4FAST_READ = 0x3C,
+    SFC_QUAD_READ = 0x3B,
+    SFC_4QUAD_READ = 0x3C,
+    SFC_RES = 0xAB,
+    SFC_JEDEC = 0x9F,
+    SFC_RDSR = 0x05,
+    SFC_WRSR2 = 0x31,
+    SFC_RDSR2 = 0x35,
+    SFC_WREN = 0x06,
+    SFC_RDNVR = 0xB5,
+    SFC_RDCR = 0x15,
+    SFC_WRNVR = 0xB1,
+    SFC_WRSR = 0x01,
+    SFC_WRSR_GIGA = 0x31,
+    SFC_RDFR = 0x48,
+    SFC_WRFR = 0x42
+} StFlashCommand_t;
+
+typedef struct access_commands {
+    StFlashCommand_t sfc_sector_erase;
+    StFlashCommand_t sfc_subsector_erase;
+    StFlashCommand_t sfc_page_program;
+    StFlashCommand_t sfc_read;
+    StFlashCommand_t sfc_fast_read;
+} flash_access_commands_t;
+
 /////////////////////////////////////////////
 //
 // Flash attributes struct
@@ -87,7 +128,7 @@ typedef struct flash_attr {
     //
     u_int32_t hw_dev_id;
     u_int32_t rev_id;
-    BinIdT    bin_id;
+    BinIdT bin_id;
 
     //
     // size:        Total size (in bytes) of all flash devices connected to
@@ -101,7 +142,7 @@ typedef struct flash_attr {
     //
     u_int32_t sector_size;
 
-    int       num_erase_blocks;         // Number of sector defs.
+    int num_erase_blocks;         // Number of sector defs.
 
     //
     // bank_size:   Different bank means a different chip sellect or gpio settings is needed when crossing
@@ -140,13 +181,19 @@ typedef struct flash_attr {
 
     u_int8_t support_sub_and_sector; // true if flash can work in both 64KB and 4KB sectors
 
+    flash_access_commands_t access_commands;
 
 } flash_attr;
 
+// Explanation for densities field:
+// Support for density X is represented by setting the log(X) bit of flash_info.densities.
+// That is, a flash related to flash_info supports density X iff ((flash_info.densities & (1 << FD_X)) != 0).
+// FD_X is the density value as it appears in the flash JEDEC ID. For example, FD_128 == 0x18 (128Mb flash).
 typedef struct flash_info {
     const char *name;
     u_int8_t vendor;
     u_int8_t type;
+    u_int32_t densities;
     int command_set;
     int erase_command;
     u_int32_t sector_size;
@@ -161,17 +208,15 @@ typedef struct flash_info {
  * Common Macros:
  */
 #define WRITE_PROTECT_CHECKS(mfl, bank_num) { \
-        if (!mfl->attr.write_protect_support) {\
-            return MFE_NOT_SUPPORTED_OPERATION;\
-        }\
-        if (bank_num >= mfl->attr.banks_num) {\
-            return MFE_FLASH_NOT_EXIST;\
-        }\
+        if (!mfl->attr.write_protect_support) { \
+            return MFE_NOT_SUPPORTED_OPERATION; \
+        } \
+        if (bank_num >= mfl->attr.banks_num) { \
+            return MFE_FLASH_NOT_EXIST; \
+        } \
 }
 #define MAX_SUBSECTOR_NUM 8
 #define MAX_SECTORS_NUM   64
 
 #endif // MFLASH_COMMON_STRUCTS_H
 
-
-
