@@ -97,8 +97,8 @@ public:
     void checkConfTlvs(const vector<TLVConf*>& tlvs, FwComponent::comps_ids_t& compsId);
     void orderConfTlvs(vector<TLVConf*>& tlvs);
     void createConf(const string& xml, vector<u_int32_t>& buff);
-    void sign(vector<u_int32_t>& buff, const string& privateKeyFile, const string& keyPairUUid);
-    void sign(vector<u_int32_t>& buff);
+    void sign(vector<u_int32_t>& buff, const string& privateKeyFile = "",
+              const string& keyPairUUid = "", const string& openssl_engine = "", const string& openssl_key_identifier = "");
     void apply(const vector<u_int8_t>& buff);
 };
 
