@@ -172,21 +172,21 @@ bool TLVConf::isFWSupported(mfile *mf, bool read_write)
 {
     bool suppRead = false, suppWrite = false;
     //printf("-D- tlv=%s\n", _name.c_str());
-    if (_target == EXP_ROM) {
-        return true;
-    }
 
     if (nvqcCom5thGen(mf, getTlvTypeBe(), suppRead, suppWrite, _maxTlvVersionSuppByFw)) {
         //Don't throw exception if we fail to run nvqc, maybe its an old fw
         return false;
     }
+// EXP_ROM TLVs are SW2SW TLVs always have support read/write and
+// FW only support to return EXP_ROM TLV version through NVQC register 
+    if (_target != EXP_ROM) {
+        if (!suppRead) {
+            return false;
+        }
 
-    if (!suppRead) {
-        return false;
-    }
-
-    if (read_write && !suppWrite) {
-        return false;
+        if (read_write && !suppWrite) {
+            return false;
+        }
     }
     return true;
 }
@@ -379,9 +379,14 @@ void TLVConf::mnva(mfile *mf, u_int8_t *buff, u_int16_t len, u_int32_t type,
 }
 
 void TLVConf::parseParamValue(string paramMlxconfigName, string valToParse, u_int32_t& val,
-                              string& strVal)
+                              string& strVal, u_int32_t index)
 {
     Param *p = findParamByMlxconfigName(paramMlxconfigName);
+    
+    if (!p) {
+        p = findParamByMlxconfigName(paramMlxconfigName + getArraySuffixByInterval(index));
+    }
+
     if (!p) {
         throw MlxcfgException("Unknown Parameter %s",
                               paramMlxconfigName.c_str());
@@ -437,11 +442,16 @@ void TLVConf::updateParamByMlxconfigName(string paramMlxconfigName, string val)
 void TLVConf::updateParamByMlxconfigName(string paramMlxconfigName, string val, u_int32_t index)
 {
     Param *p = findParamByMlxconfigName(paramMlxconfigName);
+
+    if (!p) {
+        p = findParamByMlxconfigName(paramMlxconfigName + getArraySuffixByInterval(index));
+    }
+
     if (!p) {
         throw MlxcfgException("Unknown parameter: %s",
                               paramMlxconfigName.c_str());
     }
-    p->setVal(val, index);
+    p->setVal(val, index % MAX_ARRAY_SIZE);
 }
 
 void TLVConf::updateParamByName(string paramName, string val)
