@@ -1293,9 +1293,18 @@ mlxCfgStatus MlxCfg::createConf()
     try {
         GenericCommander commander(NULL, _mlxParams.dbName);
         commander.createConf(xml, buff);
-        if (!_mlxParams.privPemFile.empty() && !_mlxParams.keyPairUUID.empty()) {
-            commander.sign(buff, _mlxParams.privPemFile, _mlxParams.keyPairUUID);
-        } else {
+        if (!_mlxParams.privPemFile.empty() &&
+            !_mlxParams.keyPairUUID.empty()) {
+            commander.sign(buff, _mlxParams.privPemFile,
+                           _mlxParams.keyPairUUID);
+        }
+        else if (!_mlxParams.opensslEngine.empty() &&
+                 !_mlxParams.opensslKeyId.empty() &&
+                 !_mlxParams.keyPairUUID.empty()) {
+            commander.sign(buff, "", _mlxParams.keyPairUUID,
+                           _mlxParams.opensslEngine, _mlxParams.opensslKeyId);
+        }
+        else {
             commander.sign(buff);
         }
     } catch (MlxcfgException& e) {
