@@ -83,6 +83,11 @@ enum WriterId {
     WRITER_ID_LAST = 0x20
 };
 
+enum RawTlvMode{
+    SET_RAW,
+    GET_RAW
+};
+
 #define VECTOR_ITERATOR(t, v, i) \
     for (vector<t>::iterator i = v.begin(); i != v.end(); ++i)
 
@@ -129,6 +134,7 @@ typedef enum {
     Device_Description
 } info_type_t;
 
+
 void dealWithSignal();
 
 MError mnvaCom5thGen(mfile *mf, u_int8_t *buff, u_int16_t len, u_int32_t tlvType, reg_access_method_t method, QueryType qT = QueryNext);
