@@ -441,7 +441,7 @@ bool Flash::read(u_int32_t addr,
     // printf("-D- read1: addr = %#x, phys_addr = %#x\n", addr, phys_addr);
     // here we set a "silent" signal handler and deal with the received signal after the read
     mft_signal_set_handling(1);
-    rc = mf_read(_mfl, phys_addr, 4, (u_int8_t*)data);
+    rc = mf_read(_mfl, phys_addr, 4, (u_int8_t*)data, false);
     deal_with_signal();
     if (rc != MFE_OK) {
         return errmsg("Flash read failed at address %s0x%x : %s",
@@ -460,7 +460,9 @@ bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char *
     if (!readWriteCommCheck(addr, len)) {
         return false;
     }
-
+    if (verbose) {
+        printf("\33[2K\r");//clear the current line
+    }
     // Much better perf for read in a single chunk. need to work on progress report though.
     bool read_in_single_chunk = true;
 
@@ -474,7 +476,7 @@ bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char *
             u_int32_t phys_addr = cont2phys(chunk_addr);
             // printf("-D- write: addr = %#x, phys_addr = %#x\n", chunk_addr, phys_addr);
             mft_signal_set_handling(1);
-            rc = mf_read(_mfl, phys_addr, chunk_size, ((u_int8_t*)data) + chunk_addr - addr);
+            rc = mf_read(_mfl, phys_addr, chunk_size, ((u_int8_t*)data) + chunk_addr - addr, verbose);
             deal_with_signal();
             if (rc != MFE_OK) {
                 return errmsg("Flash read failed at address %s0x%x : %s",
@@ -498,9 +500,9 @@ bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char *
             if (verbose) {
                 u_int32_t new_perc = (i * 100) / len;
                 if (new_perc != perc) {
-                    printf("\r%s%%%03d", message, new_perc);
+                    printf("\33[2K\r");//clear the current line
+                    printf("\r%s%d%c", message, new_perc, '%');
                     fflush(stdout);
-
                     perc = new_perc;
                 }
             }
@@ -509,7 +511,8 @@ bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char *
 
     // Report
     if (verbose) {
-        printf("\r%s%%100", message);
+        printf("\33[2K\r");//clear the current line
+        printf("Done.");
         fflush(stdout);
     }
 
@@ -631,6 +634,9 @@ bool Flash::write(u_int32_t addr,
         u_int32_t phys_addr = cont2phys(chunk_addr);
         // printf("-D- write: addr = %#x, phys_addr = %#x\n", chunk_addr, phys_addr);
         mft_signal_set_handling(1);
+        if (_cputUtilizationApplied) {
+            mf_set_cpu_utilization(_mfl, _cpuPercent);
+        }
         rc = mf_write(_mfl, phys_addr, chunk_size, p);
         deal_with_signal();
 
@@ -1016,3 +1022,9 @@ bool Flash::set_flash_working_mode(int mode)
     _curr_sector = _curr_sector & ~(_curr_sector_size - 1);
     return true;
 }
+bool Flash::set_flash_utilization(bool is_applied, int percent)
+{
+    _cputUtilizationApplied = is_applied;
+    _cpuPercent = percent;
+    return true;
+}
