@@ -31,6 +31,7 @@
  */
 
 
+
 #if !defined(UEFI_BUILD) && !defined(NO_CS_CMD)
 #include <tools_crypto/tools_md5.h>
 #endif
@@ -51,7 +52,7 @@ extern const char *g_sectNames[];
 bool Fs2Operations::FwInit()
 {
     FwInitCom();
-    memset(&_fs2ImgInfo, 0, sizeof(_fs2ImgInfo));
+    //memset(&_fs2ImgInfo, 0, sizeof(_fs2ImgInfo)); init is done as part of the struct constructor
     _fwImgInfo.fwType = FIT_FS2;
     return true;
 }
@@ -369,7 +370,7 @@ bool Fs2Operations::checkGen(u_int32_t beg, u_int32_t offs, u_int32_t& next, con
         // CRC
         Crc16 crc;
         std::vector<u_int8_t> buffv(size);
-        u_int32_t *buff = (u_int32_t*)(&(buffv[0]));
+        u_int32_t *buff = (u_int32_t *)(buffv.size() ? (&(buffv[0])) : NULL);
 
         readBufAux((*_ioAccess), offs + sizeof(gph), buff, size, pr);
 
@@ -613,8 +614,9 @@ u_int8_t Fs2Operations::FwType()
     return FIT_FS2;
 }
 
-bool Fs2Operations::FwReadData(void *image, u_int32_t *image_size)
+bool Fs2Operations::FwReadData(void *image, u_int32_t *image_size, bool verbose)
 {
+    (void)verbose;
     if (!Fs2Verify((VerifyCallBack)NULL)) {
         return false;
     }
@@ -775,8 +777,11 @@ bool Fs2Operations::Fs2IntQuery(bool readRom, bool isStripedImage)
     return true;
 }
 
-bool Fs2Operations::FwQuery(fw_info_t *fwInfo, bool readRom, bool isStripedImage)
+bool Fs2Operations::FwQuery(fw_info_t *fwInfo, bool readRom, bool isStripedImage, bool quickQuery, bool ignoreDToc, bool verbose)
 {
+    (void)quickQuery;
+    (void)ignoreDToc;
+    (void)verbose;
     if (!Fs2IntQuery(readRom, isStripedImage)) {
         return false;
     }
@@ -784,6 +789,7 @@ bool Fs2Operations::FwQuery(fw_info_t *fwInfo, bool readRom, bool isStripedImage
     memcpy(&(fwInfo->fs2_info), &(_fs2ImgInfo.ext_info), sizeof(fs2_info_t));
     //set the chipType in fwInfo
     fwInfo->fw_type = FIT_FS2;
+    fwInfo->fw_info.chip_type = CT_CONNECTX3;
     return true;
 }
 
@@ -799,7 +805,7 @@ bool Fs2Operations::getRunningFwVer()
     u_int8_t buff[CIBFW_FW_VERSION_SIZE] = {0};
     memset(&fwVer, 0, sizeof(fwVer));
     // make sure its not mellanox OS
-    if (mget_mdevs_flags(((Flash*)_ioAccess)->getMfileObj(), &mflags)) {
+    if (mget_mdevs_flags(_ioAccess->getMfileObj(), &mflags)) {
         return errmsg("Failed to get device access type");
     }
     if (mflags & MDEVS_MLNX_OS) {
@@ -818,7 +824,7 @@ bool Fs2Operations::getRunningFwVer()
         return errmsg("Unsupported chip type.");
     }
 
-    if (mread_buffer(((Flash*)_ioAccess)->getMfileObj(), fwVerBaseAddr, buff, CIBFW_FW_VERSION_SIZE) != CIBFW_FW_VERSION_SIZE) {
+    if (mread_buffer(_ioAccess->getMfileObj(), fwVerBaseAddr, buff, CIBFW_FW_VERSION_SIZE) != CIBFW_FW_VERSION_SIZE) {
         return errmsg("Failed to extract FW version from device. CR_ERROR\n");
     }
     cibfw_FW_VERSION_unpack(&fwVer, buff);
@@ -927,7 +933,8 @@ bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
     ProgressCallBack progressFunc = burnParams.progressFunc;
     ProgressCallBackEx progressFuncEx = burnParams.progressFuncEx;
     void *progressUserData = burnParams.progressUserData;
-
+    bool isCpuUtilization = burnParams.use_cpu_utilization;
+    int cpuPercent = burnParams.cpu_utilization;
     Flash  *f = (Flash*)(this->_ioAccess);
     FImage *fim = (FImage*)(imageOps._ioAccess);
 
@@ -995,7 +1002,7 @@ bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
 
     // Go ahead and burn!
     //const char* image_name = new_image_start == 0 ? "first" : "second";
-    if (!writeImageEx(progressFuncEx, progressUserData, progressFunc, 16, data8 + 16, image_size - 16)) {
+    if (!writeImageEx(progressFuncEx, progressUserData, progressFunc, 16, data8 + 16, image_size - 16, false, false, -1, 0, isCpuUtilization, cpuPercent)) {
         return false;
     }
     // Write new signature
@@ -1044,8 +1051,8 @@ bool Fs2Operations::Fs2FailSafeBurn(Fs2Operations &imageOps,
 #ifdef __WIN__
     if (!burnParams.skipCiReq) {
         int rc;
-        mf_release_semaphore(((Flash*)_ioAccess)->getMflashObj());
-        rc = wdcif_send_image_cache_request(((Flash*)_ioAccess)->getMfileObj());
+        mf_release_semaphore(_ioAccess->getMflashObj());
+        rc = wdcif_send_image_cache_request(_ioAccess->getMfileObj());
         switch (rc) {
         case WDCIF_STATUS_SUCCESS:
             burnParams.burnStatus.imageCachedSuccessfully = true;
@@ -1405,19 +1412,24 @@ bool Fs2Operations::IntegrateDevRomInImage(Fs2Operations &imageOps)
     if (fim->getBuf() == NULL) {
         return errmsg("Bad FW image buffer\n");
     }
-    // Compine the image and the rom into new daa
-    if (!UpdateRomInImage((u_int8_t*)(&new_data[0]), (u_int8_t*)(fim->getBuf()),
-                          (u_int8_t*)(&_romSect[0]), rom_size, &actual_image_size)) {
-        return errmsg("%s", err());
+
+    if (!_romSect.empty()) {
+        // Compine the image and the rom into new daa
+        if (!UpdateRomInImage((u_int8_t*)(&new_data[0]), (u_int8_t*)(fim->getBuf()),
+                              (u_int8_t*)(&_romSect[0]), rom_size, &actual_image_size)) {
+            return errmsg("%s", err());
+        }
+
+        // close old image and open new image with the rom.
+        ((FImage*)(imageOps._ioAccess))->close();
+        ((FImage*)(imageOps._ioAccess))->open((u_int32_t*)(&new_data[0]), actual_image_size);
+
+        if (!imageOps.FwVerify((VerifyCallBack)NULL) || !imageOps.Fs2IntQuery()) {
+            return errmsg("Internal error: verify/query of image after integrating ROM failed\n");
+        }
     }
-
-    // close old image and open new image with the rom.
-    ((FImage*)(imageOps._ioAccess))->close();
-    ((FImage*)(imageOps._ioAccess))->open((u_int32_t*)(&new_data[0]), actual_image_size);
-
-
-    if (!imageOps.FwVerify((VerifyCallBack)NULL) || !imageOps.Fs2IntQuery()) {
-        return errmsg("Internal error: verify/query of image after integrating ROM failed\n");
+    else {
+        printf("-W- Skipping ROM section handling (ROM section is empty)\n");
     }
 
     return true;
@@ -1768,7 +1780,7 @@ bool Fs2Operations::Fs2SetGuidsForBlank(sg_params_t& sgParam)
         //  patchGUIDsSection(u_int32_t *buf, u_int32_t ind, guid_t guids[MAX_GUIDS], int nguids)
         patchGUIDsSection(guid_sect, 16, &sgParam.userGuids[0], _fs2ImgInfo.ext_info.guid_num);
 
-        if (!((Flash*)_ioAccess)->write(guid_sect_addr[i], guid_sect + 4, _fs2ImgInfo.ext_info.guid_num * 8 + 4, true)) {
+        if (!_ioAccess->write(guid_sect_addr[i], guid_sect + 4, _fs2ImgInfo.ext_info.guid_num * 8 + 4, true)) {
             return errmsg("flash write error (%s)\n", _ioAccess->err());
         }
     }
@@ -1787,7 +1799,7 @@ bool Fs2Operations::Fs2SetGuidsForBlank(sg_params_t& sgParam)
         old_dw.range(15, 0) =  crc;
         u_int32_t new_crc_dw = CPUTO1(old_dw);
 
-        if (!((Flash*)_ioAccess)->write(IMG_CRC_OFF, &new_crc_dw, 4, true)) {
+        if (!_ioAccess->write(IMG_CRC_OFF, &new_crc_dw, 4, true)) {
             return errmsg(MLXFW_FLASH_WRITE_ERR, "flash write error (%s).", _ioAccess->err());
         }
     }
@@ -1858,7 +1870,7 @@ bool Fs2Operations::FwBurnRom(FImage *romImg, bool ignoreProdIdCheck, bool ignor
     u_int32_t cntx_image_num;
     FindAllImageStart(romImg, cntx_image_start, &cntx_image_num, _cntx_magic_pattern);
     if (cntx_image_num != 0) {
-        return errmsg("Expecting an expansion ROM image, Recieved Mellanox FW image.");
+        return errmsg("Expecting an expansion ROM image, received Mellanox FW image.");
     }
 
     if (!Fs2IntQuery()) {
@@ -1870,7 +1882,9 @@ bool Fs2Operations::FwBurnRom(FImage *romImg, bool ignoreProdIdCheck, bool ignor
         return errmsg("Unsupported device type %d", _fwImgInfo.ext_info.dev_type);
     }
 
-    if (_romSect.empty() && !IsFwSupportingRomModify(_fwImgInfo.ext_info.fw_ver)) {
+    if (_romSect.empty()
+            && !IsFwSupportingRomModify(
+                    FwVersion(createFwVersion(&_fwImgInfo.ext_info)))) {
         return errmsg("It is not allowed to burn ROM on device which has fw: %d.%d.%d that doesn't contain ROM.", _fwImgInfo.ext_info.fw_ver[0],
                       _fwImgInfo.ext_info.fw_ver[1], _fwImgInfo.ext_info.fw_ver[2]);
     }
@@ -1951,7 +1965,7 @@ bool Fs2Operations::FwDeleteRom(bool ignoreProdIdCheck, ProgressCallBack progres
         return errmsg("The FW does not contain a ROM section.");
     }
 
-    if (!IsFwSupportingRomModify(_fwImgInfo.ext_info.fw_ver)) {
+    if (!IsFwSupportingRomModify(FwVersion(createFwVersion(&_fwImgInfo.ext_info)))) {
         return errmsg("Unsupported Fw version (%d.%d.%d).", _fwImgInfo.ext_info.fw_ver[0],
                       _fwImgInfo.ext_info.fw_ver[1], _fwImgInfo.ext_info.fw_ver[2]);
     }
@@ -2089,21 +2103,21 @@ bool Fs2Operations::FwResetNvData()
     //_fs2ImgInfo.ext_info.config_sectors, _fs2ImgInfo.ext_info.config_pad, fwSectorSz,configBaseAddr, availFlashSize);
 
     //erase addresses : [configBaseAddr..AvailFlashSize]
-    ((Flash*)_ioAccess)->set_flash_working_mode(Flash::Fwm_Default);
+    _ioAccess->set_flash_working_mode(Flash::Fwm_Default);
     u_int32_t sectorSize = _ioAccess->get_sector_size();
     u_int32_t configEndAddr = availFlashSize - (_fs2ImgInfo.ext_info.config_pad * fwSectorSz);
     for (u_int32_t eraseAddr = configBaseAddr; eraseAddr < configEndAddr; eraseAddr += sectorSize) {
-        if (!((Flash*)_ioAccess)->erase_sector(eraseAddr)) {
+        if (!_ioAccess->erase_sector(eraseAddr)) {
             return errmsg("failed to erase configuration address: 0x%x. %s", eraseAddr, _ioAccess->err());
         }
         if (_fwImgInfo.actuallyFailsafe) { // erase config sectors on the other half aswell
-            ((Flash*)_ioAccess)->set_address_convertor(_fwImgInfo.cntxLog2ChunkSize, !(_fwImgInfo.imgStart != 0));
-            if (!((Flash*)_ioAccess)->erase_sector(eraseAddr)) {
-                ((Flash*)_ioAccess)->set_address_convertor(_fwImgInfo.cntxLog2ChunkSize, (_fwImgInfo.imgStart != 0));;
+            _ioAccess->set_address_convertor(_fwImgInfo.cntxLog2ChunkSize, !(_fwImgInfo.imgStart != 0));
+            if (!_ioAccess->erase_sector(eraseAddr)) {
+                _ioAccess->set_address_convertor(_fwImgInfo.cntxLog2ChunkSize, (_fwImgInfo.imgStart != 0));;
                 return errmsg("failed to erase configuration address: 0x%x. %s", eraseAddr, _ioAccess->err());
             }
             // restore address converter
-            ((Flash*)_ioAccess)->set_address_convertor(_fwImgInfo.cntxLog2ChunkSize, (_fwImgInfo.imgStart != 0));
+            _ioAccess->set_address_convertor(_fwImgInfo.cntxLog2ChunkSize, (_fwImgInfo.imgStart != 0));
         }
     }
     return true;
