@@ -47,13 +47,16 @@ def insertField(val1, start1, val2, start2, size):
 class MtcrException(Exception):
     pass
 
+class CmdIfException(Exception):
+    pass
+
 ##########################
 CMTCR = None
 try:
     from ctypes import *
     ctypes.CDLL._func_restype_ = ctypes.c_ulonglong
     if platform.system() == "Windows" or os.name == "nt":
-        CMTCR = CDLL("libmtcr-1.dll", use_errno=True)
+        CMTCR = CDLL(".\\libmtcr-1.dll", use_errno=True)
     else:
         try:
             CMTCR = CDLL("cmtcr.so", use_errno=True)
@@ -172,9 +175,9 @@ if CMTCR:
 else:
     import subprocess
     import string
-
+    
     def getstatusoutput(cmd):
-        pipe = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+        pipe = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
         cout = pipe.stdout
         output = cout.read()
         cout.close()
@@ -201,11 +204,11 @@ else:
 
         ##########################
         def read4(self, addr):
-            cmd = "mcra %s 0x%x" % (self.dev, addr)
+            cmd = ["mcra", self.dev, hex(addr)]
             rc, out = getstatusoutput(cmd)
             if rc:
                 raise MtcrException("Failed to read from mst device from address 0x%x: %s" % (addr, out))
-            return string.atoi(out, 16)
+            return int(out, 16)
 
         ##########################
         def readField(self, addr, startBit, size):
@@ -213,7 +216,7 @@ else:
 
         ##########################
         def write4(self, addr, val):
-            cmd = "mcra %s 0x%x 0x%x" % (self.dev, addr, val)
+            cmd = ["mcra", self.dev, hex(addr), hex(val)]
             rc, out = getstatusoutput(cmd)
             if rc:
                 raise MtcrException("Failed to wrtie to mst device to address 0x%x: %s" % (addr, out))
