@@ -64,7 +64,8 @@ try:
     from mlxfwresetlib.mlxfwreset_status_checker import FirmwareResetStatusChecker
     from mlxfwresetlib.logger import LoggerFactory
     from mlxfwresetlib.cmd_reg_mfrl import CmdRegMfrl
-    from mlxfwresetlib.cmd_reg_mpcir import CmdRegMpcir
+    # from mlxfwresetlib.cmd_reg_mpcir import CmdRegMpcir
+    from mlxfwresetlib.cmd_reg_mcam import CmdRegMcam
     if os.name != 'nt':
         if not getattr(sys, 'frozen', False):
             sys.path.append(os.sep.join((os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "mlxpci")))
@@ -80,6 +81,10 @@ except Exception as e:
     sys.exit(1)
 
 # Constants ###########################
+class SyncOwner():
+    TOOL = 0
+    DRIVER = 1
+
 MLNX_DEVICES = [
                dict(name="ConnectX2", devid=0x190),
                dict(name="ConnectX3Pro", devid=0x1f7),
@@ -89,16 +94,19 @@ MLNX_DEVICES = [
                dict(name="ConnectX5", devid=0x20d, status_config_not_done=(0xb5e04, 31)),
                dict(name="BlueField", devid=0x211, status_config_not_done=(0xb5e04, 31)),
                dict(name="BlueField2", devid=0x214, status_config_not_done=(0xb5e04, 31)),
+               dict(name="BlueField3", devid=0x21c, status_config_not_done=(0xb5e04, 31)),
                dict(name="ConnectX6", devid=0x20f, status_config_not_done=(0xb5f04, 31)),
                dict(name="ConnectX6DX", devid=0x212, status_config_not_done=(0xb5f04, 31)),
                dict(name="ConnectX6LX", devid=0x216, status_config_not_done=(0xb5f04, 31)),
+               dict(name="ConnectX7", devid=0x218, status_config_not_done=(0xb5f04, 31)),
                dict(name="ConnectX3", devid=0x1f5),
                dict(name="SwitchX", devid=0x245),
                dict(name="IS4", devid=0x1b3),
                ]
 
 # Supported devices.
-SUPP_DEVICES = ["ConnectIB", "ConnectX4", "ConnectX4LX", "ConnectX5", "BlueField", "ConnectX6", "ConnectX6DX", "ConnectX6LX", "BlueField2"]
+SUPP_DEVICES = ["ConnectIB", "ConnectX4", "ConnectX4LX", "ConnectX5", "BlueField",
+                "ConnectX6", "ConnectX6DX", "ConnectX6LX", "BlueField2", "ConnectX7", "BlueField3"]
 SUPP_OS = ["FreeBSD", "Linux", "Windows"]
 
 IS_MSTFLINT = True
@@ -157,7 +165,7 @@ device_global = None
 pciModuleName      = "mst_ppc_pci_reset.ko"
 pathToPciModuleDir = "/etc/mft/mlxfwreset"
 
-SUPPORTED_DEVICES_WITH_SWITCHES = ["15b3", "10ee"]
+SUPPORTED_DEVICES_WITH_SWITCHES = ["15b3", "10ee","1af4"] #1af4 for Red Hat (virtio)
 
 
 ######################################################################
@@ -167,7 +175,6 @@ SUPPORTED_DEVICES_WITH_SWITCHES = ["15b3", "10ee"]
 class PcnrError(Exception):
     pass
 
-
 ######################################################################
 # Description:  NoError Exception
 # OS Support :  Linux/FreeBSD/Windows.
@@ -175,6 +182,13 @@ class PcnrError(Exception):
 class NoError(Exception):
     pass
 
+######################################################################
+# Description:  Warning Exception
+# OS Support :  Linux/FreeBSD/Windows.
+######################################################################
+class WarningException(Exception):
+    pass
+
 ######################################################################
 # Description:  is_fw_ready
 # OS Support :  Linux/FreeBSD/Windows.
@@ -785,12 +799,17 @@ class MlnxPciOpFreeBSD(MlnxPciOp):
         if(linesCount > 1):
             raise RuntimeError("Found more than one secbus with the value: %s" %busNum)
         hostBridge = "pcib%s" %(out.split("pcib.")[1].split(".")[0])
+        
         # get pci address of hostBridge
-        cmd = "pciconf -l | grep %s | cut -f 1 | cut -f 2 -d@ | sed -e 's/\(.*\):\s*/\\1/g'" % hostBridge
-        (rc, bridgeDevAddr, _) = cmdExec(cmd)
-        if rc != 0 or len(bridgeDevAddr.strip()) == 0:
-            raise RuntimeError("Failed to extract pci bridge address")
-        return bridgeDevAddr.strip()
+        cmd = "pciconf -l | grep {0}".format(hostBridge)
+        rc, out, _ = cmdExec(cmd)
+        if rc != 0:
+            raise RuntimeError("Failed to extract PCI bridge address ({0})".format(cmd))
+        try:
+            bridgeDevAddr = out.split()[0].split('@')[1][:-1] # Extract the pci device from the output
+        except:
+            raise RuntimeError("Failed to extract PCI bridge address ({0})".format(out))
+        return bridgeDevAddr
 
     def getPciECapAddr(self, bridgeDev):
         cmd = "pciconf -lc %s | grep \"cap 10\" | sed -e 's/.*\[\([0-9,a-f]*\)\].*/\\1/g'" % bridgeDev
@@ -819,7 +838,7 @@ class MlnxPciOpFreeBSD(MlnxPciOp):
     def loadPCIConfigurationSpace(self, devAddr, pci_device, full=True):
         pci_device.restore_configuration_space()
 
-
+# TODO are we using this class ??? 
 class MlnxPciOpWindows(MlnxPciOp):
 
     def __init__(self):
@@ -882,7 +901,7 @@ def mstRestart(busId):
     for _ in range(0, 30):
         (rc, stdout, _) = cmdExec(cmd)
         if rc != 0:
-            raise NoError("Failed to get the status of the mst module (RC=%d), please restart MST manually" % rc)
+            raise WarningException("Failed to get the status of the mst module, please restart MST manually")
         matchedLines = [line for line in stdout.split('\n') if "mst_pciconf" in line]
         logger.debug('matchedLines = {0}'.format(matchedLines))
         if len(matchedLines) == 0:
@@ -904,7 +923,7 @@ def mstRestart(busId):
         else :
             time.sleep(2)
     if rc != 0:
-        raise NoError("Failed to run lspci command (RC=%d), please restart MST manually" % rc)
+        raise WarningException("Failed to run lspci command, please restart MST manually")
     if foundBus == False:
         raise RuntimeError("The device is not appearing in lspci output!")
 
@@ -914,7 +933,7 @@ def mstRestart(busId):
     (rc, stdout, stderr) = cmdExec(cmd)
     if rc != 0:
         logger.debug('stdout:\n{0}\nstderr:\n{1}'.format(stdout,stderr))
-        raise NoError("Failed to restart MST (RC=%d), please restart MST manually" % rc)
+        raise WarningException("Failed to restart MST, please restart MST manually")
     set_signal_handler()
     return 0
 
@@ -1077,8 +1096,9 @@ def disablePci(bridgeDev, capAddr):
     width = "L"
     oldCapDw = PciOpsObj.read(bridgeDev, disableAddr, width)
     # turn on Link Disable bit
-    logger.debug('Disable PCI bridge address {0}'.format(bridgeDev))
+    logger.debug('Disable PCI bridge {0} Link Control Register address {1:x}'.format(bridgeDev, disableAddr))
     newCapDw = oldCapDw | 0x10
+    logger.debug('old value: {0:x}; new value: {1:x}'.format(oldCapDw,newCapDw))
     PciOpsObj.write(bridgeDev, disableAddr, newCapDw, width)
 
     return (bridgeDev, disableAddr, oldCapDw, width)
@@ -1107,17 +1127,17 @@ def resetPciAddr(device,devicesSD,driverObj, cmdLineArgs):
         if isConnectedToMellanoxSwitchDevice():
             pci_device_bridge = PciOpsObj.getPciBridgeAddr(DevDBDF)
             pci_device_to_poll_devid = PciOpsObj.getPciBridgeAddr(pci_device_bridge)
-            pci_device_to_poll_dll_link_active = PCIDeviceFactory().get(pci_device_to_poll_devid, "debug")
+            root_pci_device = PCIDeviceFactory().get(pci_device_to_poll_devid, "debug")
         elif isPPC:
             pci_device_to_poll_devid = DevDBDF
         else:
             pci_device_to_poll_devid = DevDBDF
-            pci_device_to_poll_dll_link_active = PCIDeviceFactory().get(PciOpsObj.getPciBridgeAddr(DevDBDF), "debug")
+            root_pci_device = PCIDeviceFactory().get(PciOpsObj.getPciBridgeAddr(DevDBDF), "debug")
 
 
         logger.debug('pci_device_to_poll_devid={0}'.format(pci_device_to_poll_devid))
         if not isPPC:
-            logger.debug('pci_device_to_poll_dll_link_active={0}'.format(pci_device_to_poll_dll_link_active))
+            logger.debug('root_pci_device={0}'.format(root_pci_device))
 
 
     if isWindows is False: # relevant for ppc(p7) TODO check power8/9 shouldn't use mst-save/load
@@ -1167,6 +1187,20 @@ def resetPciAddr(device,devicesSD,driverObj, cmdLineArgs):
     if SkipMultihostSync or not CmdifObj.isMultiHostSyncSupported():
         stopDriver(driverObj)
     else:
+        # Isolated Smart-NIC support
+        try:
+            cmd = 'echo 0xde31001 > /sys/bus/platform/drivers/mlx-bootctl/fw_reset'
+            rc, _, _ = cmdExec(cmd)
+            if rc == 0:
+                logger.debug('GPIO signal was sent from Smart-NIC integrated ARM')
+                # For debug: if signal sent ok, out should be 0x4:
+                # cmd = 'mcra <dbdf> 0xf3404'
+                # rc, out, _ = cmdExec(cmd)
+                # logger.debug("read val of yu.gpio.gpios_bits.gpio_internal.datain = " + str(out))
+            else:
+                raise Exception
+        except:
+            logger.debug('Not inside Smart-NIC integrated ARM or command is not supported in FW')
         stopDriverSync(driverObj)
 
     logger.debug('start critical time (driver is unloaded)')
@@ -1231,40 +1265,55 @@ def resetPciAddr(device,devicesSD,driverObj, cmdLineArgs):
                 PciOpsObj.waitForDevice(pciDev)
 
         # Poll DLL Link Active (required to support AMD - RM #1599465)
-        if not isPPC and pci_device_to_poll_dll_link_active.dll_link_active_reporting_capable:
+        if not isPPC and root_pci_device.dll_link_active_reporting_capable:
             MAX_WAIT_TIME = 2  # sec
             poll_start_time = time.time()
             for counter in range(1000*MAX_WAIT_TIME):
-                if pci_device_to_poll_dll_link_active.dll_link_active == 1:
+                if root_pci_device.dll_link_active == 1:
                     poll_end_time = (time.time() - poll_start_time) * 1000
                     logger.debug('DLL link active is ready after {0} msec'.format(poll_end_time))
                     break
                 time.sleep(0.001)
             else:
-                print("-W- DLL link is not active after {0} sec (PCI device {1})".format(
-                    MAX_WAIT_TIME,pci_device_to_poll_dll_link_active.dbdf))
+                print("\n-W- DLL link is not active (PCI device {0})".format(root_pci_device.dbdf))
 
         # Wait for FW (Iron) - Read devid
         MAX_WAIT_TIME = 2 # sec
         poll_start_time = time.time()
         for cntr in range(1000*MAX_WAIT_TIME):
-            if PciOpsObj.read(pci_device_to_poll_devid, 0) != 0xffffffff:
+            if PciOpsObj.read(pci_device_to_poll_devid, 0)>>16 != 0xffff: # device-id (0x2.W)
                 iron_end_time = (time.time() - poll_start_time) * 1000
                 logger.debug('IRON is ready after {0} msec'.format(iron_end_time))
                 break
             time.sleep(0.001)
         else:
             print("-W- FW is not ready after {0} sec".format(MAX_WAIT_TIME))
+        # Restore PCI configuration
+        #############################
+        logger.debug('Restore PCI configuration')
+        if isPPC:
+            logger.debug('PPC : indication for re-enumeration by MSE bit on each PCI device')
+            for pciDev in devList+devListsSD:
+                pci_device_object = pci_device_dict[pciDev]
+                command_reg = PciOpsObj.read(pciDev, COMMAND_ADDR, "W")
+                logger.debug('command_reg for [{0}]is {1:x}.'.format(pciDev, command_reg))
+                if (command_reg & 0x2) == 0: # command_reg[MSE] - Indication for re-enumeration (OS rescan)
+                    PciOpsObj.loadPCIConfigurationSpace(pciDev, pci_device_object, full=True)
+                else:
+                    PciOpsObj.loadPCIConfigurationSpace(pciDev, pci_device_object, full=False)
+        else:
+            logger.debug('Indication for re-enumeration is by HotPlug on bridge')
+            to_restore_pci_conf = not (root_pci_device.hotplug_capable and root_pci_device.hotplug_interrupt_enable)
+            logger.debug('hotplug_capable={0}'.format(root_pci_device.hotplug_capable))
+            logger.debug('hotplug_interrupt_enable={0}'.format(root_pci_device.hotplug_interrupt_enable))
+            logger.debug('to_restore_pci_conf={0}'.format(to_restore_pci_conf))
+            for pciDev in devList+devListsSD:
+                pci_device_object = pci_device_dict[pciDev]
+                if to_restore_pci_conf:
+                    PciOpsObj.loadPCIConfigurationSpace(pciDev, pci_device_object, full=True)
+                else: # os re-enumeration
+                    PciOpsObj.loadPCIConfigurationSpace(pciDev, pci_device_object, full=False)
 
-        # Restore PCI configuration (including socket direct devices)
-        for pciDev in devList+devListsSD:
-            pci_device_object = pci_device_dict[pciDev]
-            command_reg = PciOpsObj.read(pciDev, COMMAND_ADDR, "W")
-            logger.debug('command_reg for [{0}]is {1:x}.'.format(pciDev, command_reg))
-            if (command_reg & 0x2) == 0: # command_reg[MSE] - Indication for re-enumeration (OS rescan)
-                PciOpsObj.loadPCIConfigurationSpace(pciDev, pci_device_object, full=True)
-            else:
-                PciOpsObj.loadPCIConfigurationSpace(pciDev, pci_device_object, full=False)
 
     # Need to re-open the file handler because PCI tree is updated (OS remove/rescan)
     if isWindows is False:
@@ -1300,14 +1349,13 @@ def isConnectedToMellanoxSwitchDevice():
         return False
 
 
-def send_reset_cmd_to_fw(mfrl, reset_level, reset_type):
+def send_reset_cmd_to_fw(mfrl, reset_level, reset_type, reset_sync=SyncOwner.TOOL):
     try:
         printAndFlush("-I- %-40s-" % ("Sending Reset Command To Fw"), endChar="")
         logger.debug('[Timing Test] MFRL')
-        mfrl.send(reset_level, reset_type)
+        mfrl.send(reset_level, reset_type, reset_sync)
         printAndFlush("Done")
     except Exception as e:
-        printAndFlush("Failed")
         raise e
 
 
@@ -1331,7 +1379,7 @@ def sendResetToFWSync(mfrl, reset_level, reset_type):
             send_reset_cmd_to_fw(mfrl, reset_level, reset_type)
         except Exception as e:
             CmdifObj.multiHostSync(SYNC_STATE_IDLE, SYNC_TYPE_FW_RESET)
-            raise e
+            raise RuntimeError("Command is not supported")
 
         # Socket Direct - send SYNC_STATE_GET_READY for all other devices in SD
         for CmdifObjSD in CmdifObjsSD:
@@ -1458,7 +1506,17 @@ def resetFlow(device, devicesSD, reset_level, reset_type, cmdLineArgs, mfrl):
     logger.info('resetFlow() called')
 
     all_devices = [device] + devicesSD
-    driverObj = MlnxDriverFactory().getDriverObj(logger, skipDriver, all_devices)
+
+
+    # A temporary solution to get the pci-device-id (required for SmartNic)
+    # This solution is not good when the user insert device in PCI address format (-d xx:xx.x)
+    pci_device_id = None
+    if platform.system() == "Windows" or os.name == "nt":
+        if '_' in device_global:
+            pci_device_id = int(device_global.split('_')[0][2:])
+
+
+    driverObj = MlnxDriverFactory().getDriverObj(logger, skipDriver, all_devices, pci_device_id)
 
     try:
         isPpc64 = ("ppc64" in platform.machine())
@@ -1553,16 +1611,21 @@ def rebootMachine():
 ######################################################################
 # Description:  execute reset level for device
 ######################################################################
-def execResLvl(device, devicesSD, reset_level, reset_type, cmdLineArgs, mfrl, mpcir):
+def execResLvl(device, devicesSD, reset_level, reset_type, reset_sync, cmdLineArgs, mfrl):
     
-    if mfrl.is_phy_less_reset(reset_type):
-        logger.debug('[Timing Test] MPCIR')
-        mpcir.prepare_for_phyless_fw_upgrade()
+    # mpcir usage removed due to RM #2214400
+    # # In new FW, the FW doesn't need this command any more
+    # if mfrl.is_phy_less_reset(reset_type):
+    #     logger.debug('[Timing Test] MPCIR')
+    #     mpcir.prepare_for_phyless_fw_upgrade()
 
     if reset_level == mfrl.LIVE_PATCH:
         send_reset_cmd_to_fw(mfrl, reset_level, reset_type)
     elif reset_level == mfrl.PCI_RESET:
-        resetFlow(device, devicesSD, reset_level, reset_type, cmdLineArgs, mfrl)
+        if reset_sync == SyncOwner.DRIVER:
+            send_reset_cmd_to_fw(mfrl, reset_level, reset_type, reset_sync)
+        else:
+            resetFlow(device, devicesSD, reset_level, reset_type, cmdLineArgs, mfrl)
     elif reset_level == mfrl.WARM_REBOOT:
         send_reset_cmd_to_fw(mfrl, reset_level, reset_type)
         rebootMachine()
@@ -1655,6 +1718,12 @@ def main():
                         choices=CmdRegMfrl.reset_types(),
                         dest='reset_type',
                         help='Run reset with the specified reset-type')
+    options_group.add_argument('--sync',
+                        type=int,
+                        choices=[SyncOwner.TOOL, SyncOwner.DRIVER],
+                        default=SyncOwner.TOOL,
+                        dest='reset_sync',
+                        help='Run reset with the specified reset-sync')
     options_group.add_argument('--yes',
                         '-y',
                         help='answer "yes" on prompt',
@@ -1686,7 +1755,7 @@ def main():
                                help=argparse.SUPPRESS)
     options_group.add_argument('--skip_fsm_sync',
                                action="store_true",
-                               help="Skip fsm syncing")
+                               help=argparse.SUPPRESS)
 
     # Skip on the self discover devices logic for socket direct(option to disable the feature in case of bug)
     options_group.add_argument('--skip_socket_direct',
@@ -1769,19 +1838,28 @@ def main():
     skipDriver = args.skip_driver
     SkipMultihostSync = args.skip_fsm_sync
     MstDevObj = mtcr.MstDevice(device)
-    RegAccessObj = regaccess.RegAccess(MstDevObj)
+    RegAccessObj = regaccess.RegAccess(MstDevObj)    
     CmdifObj = cmdif.CmdIf(MstDevObj)
     PciOpsObj = MlnxPciOpFactory().getPciOpObj(DevDBDF)
 
 
     mfrl = CmdRegMfrl(RegAccessObj)
-    mpcir = CmdRegMpcir(RegAccessObj)
+    # mpcir = CmdRegMpcir(RegAccessObj)
+    mcam = CmdRegMcam(RegAccessObj)
 
     logger.info('Check if device is livefish')
     DevMgtObj = dev_mgt.DevMgt(MstDevObj) # check if device is in livefish
     if DevMgtObj.isLivefishMode() == 1:
         raise RuntimeError("%s is not supported for device in Flash Recovery mode" % PROG)
 
+    # Check if other process is accessing the device (burning the device)
+    # Supportted on Windows OS only
+    if platform.system() == "Windows":
+        from tools_sync import ToolsSync
+        if ToolsSync(MstDevObj.mf).lock() == False:
+            raise RuntimeError("Other tool is accessing the device! Please try again latter")    
+
+
     # Socket Direct - Create a list of command i/f for all "other" devices
     global CmdifObjsSD
     global MstDevObjsSD
@@ -1828,11 +1906,14 @@ def main():
             raise re
 
     print("")
+
     if command == "query":
         print(mfrl.query_text())
+        print(mcam.reset_sync_query_text())
+
     elif command == "reset":
 
-        # print("Reset device {0}:".format(device))
+        #print("Reset device {0}:".format(device))
 
         reset_level = mfrl.default_reset_level() if args.reset_level is None else args.reset_level
         # print("  * reset-level is '{0}' ({1})".format(reset_level, mfrl.reset_level_description(reset_level)))
@@ -1847,12 +1928,18 @@ def main():
         if mfrl.is_default_reset_type(reset_type) is False and mfrl.is_reset_level_support_reset_type(reset_level) is False:
             raise RuntimeError("Reset-level '{0}' is not supported with reset-type '{1}'".format(reset_level, reset_type))
 
+        reset_sync = args.reset_sync
+        if reset_sync == SyncOwner.DRIVER and mcam.is_reset_by_fw_driver_sync_supported() is False:
+            raise RuntimeError("Synchronization by driver is not supported for this device")
+        if reset_sync != SyncOwner.TOOL and reset_level != CmdRegMfrl.PCI_RESET:
+            raise RuntimeError("Reset-sync '{0}' is not supported with reset-level '{1}'".format(reset_sync, reset_level))
+
         minimal_or_requested = 'Minimal' if args.reset_level is None else 'Requested'
         print("{0} reset level for device, {1}:\n".format(minimal_or_requested , device))
         print("{0}: {1}".format(reset_level,mfrl.reset_level_description(reset_level)))
 
         AskUser("Continue with reset", yes)
-        execResLvl(device, devicesSD, reset_level, reset_type, args, mfrl, mpcir)
+        execResLvl(device, devicesSD, reset_level, reset_type, reset_sync, args, mfrl)
         if reset_level != CmdRegMfrl.COLD_REBOOT and reset_level != CmdRegMfrl.WARM_REBOOT:
             if FWResetStatusChecker.GetStatus() == FirmwareResetStatusChecker.FirmwareResetStatusFailed:
                 reset_fsm_register()
@@ -1872,6 +1959,8 @@ if __name__ == '__main__':
         rc = main()
     except NoError as e:
         print("-I- %s." % str(e))
+    except WarningException as e:
+        print("-W- %s." % str(e))
     except Exception as e:
         print("-E- %s." % str(e))
         rc = 1
