@@ -146,6 +146,11 @@ static struct device_sem_info g_dev_sem_info_db[] = {
         {0xe74e0},               // hw_sem_addr
         1,                       // vsec_sem_supported
     },
+    {
+        DeviceBlueField3,        // dev_id
+        {0xe74e0},               // hw_sem_addr
+        1,                       // vsec_sem_supported
+    },
     {
         DeviceQuantum,          // dev_id
         {0xa68f8},               // hw_sem_addr
@@ -173,7 +178,7 @@ static struct device_sem_info g_dev_sem_info_db[] = {
     },
     {
         DeviceConnectX7,         // dev_id
-        {0xe74e0},               // hw_sem_addr
+        {0xe5660},               // hw_sem_addr
         1,                       // vsec_sem_supported
     },
     {
@@ -198,7 +203,7 @@ static struct device_sem_info g_dev_sem_info_db[] = {
     },
     {
         DeviceSpectrum4,         // dev_id
-        {0xe74e0},               // hw_sem_addr
+        {0xa68f8},               // hw_sem_addr
         1,                       // vsec_sem_supported
     },
 };
